@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006-2007 Brian Tarricone, <bjt23@cornell.edu>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,12 +30,17 @@
 #include <gobject/gmarshal.h>
 
 #include "xfdesktop-icon.h"
+#include "xfdesktop-marshal.h"
 
 struct _XfdesktopIconPrivate
 {
     gint16 row;
     gint16 col;
-    GdkRectangle extents;
+
+    gboolean extents_dirty;
+    GdkRectangle pixbuf_extents;
+    GdkRectangle text_extents;
+    GdkRectangle total_extents;
 };
 
 enum {
@@ -44,12 +49,9 @@ enum {
     SIG_POS_CHANGED,
     SIG_SELECTED,
     SIG_ACTIVATED,
-    SIG_MENU_POPUP,
     SIG_N_SIGNALS,
 };
 
-static void xfdesktop_icon_class_init(XfdesktopIconClass *klass);
-static void xfdesktop_icon_init(XfdesktopIcon *icon);
 
 static guint __signals[SIG_N_SIGNALS] = { 0, };
 
@@ -57,38 +59,6 @@ static guint __signals[SIG_N_SIGNALS] = { 0, };
 G_DEFINE_ABSTRACT_TYPE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
 
 
-static void
-xfdesktop_icon_marshal_BOOLEAN__VOID(GClosure *closure,
-                                     GValue *return_value,
-                                     guint n_param_values,
-                                     const GValue *param_values,
-                                     gpointer invocation_hint,
-                                     gpointer marshal_data)
-{
-    typedef gboolean (*GMarshalFunc_BOOLEAN__VOID)(gpointer, gpointer);
-    register GMarshalFunc_BOOLEAN__VOID callback;
-    register GCClosure *cc = (GCClosure *)closure;
-    register gpointer data1, data2;
-    gboolean ret = FALSE;
-    
-    g_return_if_fail(return_value);
-    g_return_if_fail(n_param_values == 1);
-    
-    if(G_CCLOSURE_SWAP_DATA(closure)) {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer(param_values + 0);
-    } else {
-        data1 = g_value_peek_pointer(param_values + 0);
-        data2 = closure->data;
-    }
-
-    callback = (GMarshalFunc_BOOLEAN__VOID)(marshal_data ? marshal_data : cc->callback);
-
-    ret = callback(data1, data2);
-    
-    g_value_set_boolean(return_value, ret);
-}
-
 static void
 xfdesktop_icon_class_init(XfdesktopIconClass *klass)
 {
@@ -137,17 +107,8 @@ xfdesktop_icon_class_init(XfdesktopIconClass *klass)
                                                             activated),
                                             g_signal_accumulator_true_handled,
                                             NULL,
-                                            xfdesktop_icon_marshal_BOOLEAN__VOID,
+                                            xfdesktop_marshal_BOOLEAN__VOID,
                                             G_TYPE_BOOLEAN, 0);
-    
-    __signals[SIG_MENU_POPUP] = g_signal_new("menu-popup",
-                                             XFDESKTOP_TYPE_ICON,
-                                             G_SIGNAL_RUN_LAST,
-                                             G_STRUCT_OFFSET(XfdesktopIconClass,
-                                                             menu_popup),
-                                             NULL, NULL,
-                                             g_cclosure_marshal_VOID__VOID,
-                                             G_TYPE_NONE, 0);
 }
 
 static void
@@ -155,6 +116,7 @@ xfdesktop_icon_init(XfdesktopIcon *icon)
 {
     icon->priv = G_TYPE_INSTANCE_GET_PRIVATE(icon, XFDESKTOP_TYPE_ICON,
                                              XfdesktopIconPrivate);
+    icon->priv->extents_dirty = TRUE;
 }
 
 
@@ -186,25 +148,46 @@ xfdesktop_icon_get_position(XfdesktopIcon *icon,
 
 void
 xfdesktop_icon_set_extents(XfdesktopIcon *icon,
-                           const GdkRectangle *extents)
+                           const GdkRectangle *pixbuf_extents,
+                           const GdkRectangle *text_extents,
+                           const GdkRectangle *total_extents)
 {
-    g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-    
-    memcpy(&icon->priv->extents, extents, sizeof(GdkRectangle));
+    g_return_if_fail(XFDESKTOP_IS_ICON(icon) && pixbuf_extents
+                     && text_extents && total_extents);
+
+    icon->priv->pixbuf_extents = *pixbuf_extents;
+    icon->priv->text_extents = *text_extents;
+    icon->priv->total_extents = *total_extents;
+    icon->priv->extents_dirty = FALSE;
 }
 
 gboolean
 xfdesktop_icon_get_extents(XfdesktopIcon *icon,
-                           GdkRectangle *extents)
+                           GdkRectangle *pixbuf_extents,
+                           GdkRectangle *text_extents,
+                           GdkRectangle *total_extents)
 {
-    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon) && extents, FALSE);
-    
-    if(icon->priv->extents.width > 0 && icon->priv->extents.height > 0) {
-        memcpy(extents, &icon->priv->extents, sizeof(GdkRectangle));
-        return TRUE;
-    }
-    
-    return FALSE;
+    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), FALSE);
+
+    if(icon->priv->extents_dirty)
+        return FALSE;
+
+    if(pixbuf_extents)
+        *pixbuf_extents = icon->priv->pixbuf_extents;
+    if(text_extents)
+        *text_extents = icon->priv->text_extents;
+    if(total_extents)
+        *total_extents = icon->priv->total_extents;
+
+    return TRUE;
+}
+
+void
+xfdesktop_icon_mark_extents_dirty(XfdesktopIcon *icon)
+{
+    g_return_if_fail(XFDESKTOP_IS_ICON(icon));
+
+    icon->priv->extents_dirty = TRUE;
 }
 
 
@@ -214,9 +197,10 @@ GdkPixbuf *
 xfdesktop_icon_peek_pixbuf(XfdesktopIcon *icon,
                            gint size)
 {
-    XfdesktopIconClass *klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
     g_return_val_if_fail(klass->peek_pixbuf, NULL);
     
     return klass->peek_pixbuf(icon, size);
@@ -226,9 +210,10 @@ xfdesktop_icon_peek_pixbuf(XfdesktopIcon *icon,
 G_CONST_RETURN gchar *
 xfdesktop_icon_peek_label(XfdesktopIcon *icon)
 {
-    XfdesktopIconClass *klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
     g_return_val_if_fail(klass->peek_label, NULL);
     
     return klass->peek_label(icon);
@@ -238,9 +223,12 @@ xfdesktop_icon_peek_label(XfdesktopIcon *icon)
 GdkDragAction
 xfdesktop_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
-    XfdesktopIconClass *klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), FALSE);
+    
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    
     if(!klass->get_allowed_drag_actions)
         return 0;
     
@@ -251,9 +239,12 @@ xfdesktop_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 GdkDragAction
 xfdesktop_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
 {
-    XfdesktopIconClass *klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), FALSE);
+    
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    
     if(!klass->get_allowed_drop_actions)
         return 0;
     
@@ -266,9 +257,10 @@ xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
                             XfdesktopIcon *src_icon,
                             GdkDragAction action)
 {
-    XfdesktopIconClass *klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), FALSE);
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);    
     g_return_val_if_fail(klass->do_drop_dest, FALSE);
     
     return klass->do_drop_dest(icon, src_icon, action);
@@ -278,10 +270,12 @@ xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
 G_CONST_RETURN gchar *
 xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon)
 {
-    XfdesktopIconClass *klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
     
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    
     if(!klass->peek_tooltip)
         return NULL;
     
@@ -289,17 +283,20 @@ xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon)
 }
 
 /*< optional >*/
-GtkWidget *
-xfdesktop_icon_get_popup_menu(XfdesktopIcon *icon)
+gboolean
+xfdesktop_icon_populate_context_menu(XfdesktopIcon *icon,
+                                     GtkWidget *menu)
 {
-    XfdesktopIconClass *klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    XfdesktopIconClass *klass;
     
-    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), FALSE);
     
-    if(!klass->get_popup_menu)
-        return NULL;
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
     
-    return klass->get_popup_menu(icon);
+    if(!klass->populate_context_menu)
+        return FALSE;
+    
+    return klass->populate_context_menu(icon, menu);
 }
 
 GtkWidget *
@@ -315,6 +312,7 @@ void
 xfdesktop_icon_pixbuf_changed(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
+    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(icon, __signals[SIG_PIXBUF_CHANGED], 0);
 }
 
@@ -322,6 +320,7 @@ void
 xfdesktop_icon_label_changed(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
+    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(icon, __signals[SIG_LABEL_CHANGED], 0);
 }
 
@@ -329,6 +328,7 @@ void
 xfdesktop_icon_position_changed(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
+    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(icon, __signals[SIG_POS_CHANGED], 0);
 }
 
@@ -337,6 +337,7 @@ void
 xfdesktop_icon_selected(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
+    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(G_OBJECT(icon), __signals[SIG_SELECTED], 0, NULL);
 }
 
@@ -351,10 +352,3 @@ xfdesktop_icon_activated(XfdesktopIcon *icon)
     
     return ret;
 }
-
-void
-xfdesktop_icon_menu_popup(XfdesktopIcon *icon)
-{
-    g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-    g_signal_emit(G_OBJECT(icon), __signals[SIG_MENU_POPUP], 0, NULL);
-}
